@@ -22,9 +22,9 @@ import importlib.util
 
 
 # Optionally import imgui
-imgui_loader = importlib.util.find_spec('imgui')
+imgui_loader = importlib.util.find_spec('slimgui')
 if imgui_loader is not None:
-    import imgui
+    from slimgui import imgui
     # from imgui.integrations.pyglet import create_renderer
 else:
     imgui = None
@@ -85,7 +85,16 @@ def preprocess_guiparams(_gui_params):
             gui_params[name]['__key__'] = key
         else:
             if isinstance(val, (tuple, list)):
-                val, opts = val
+                if len(val) == 2:
+                    val, opts = val
+                else:
+                    # Allow a user to also specify paraemters as sequences
+                    seq = val[1:]
+                    val = val[0]
+                    if len(seq) == 2:
+                        opts = {'min': seq[0], 'max': seq[1]}
+                    else:
+                        raise ValueError('Currently unsupported option sequence')
             else:
                 opts = {}
             opts['__key__'] = key
@@ -203,10 +212,15 @@ if imgui is not None:
         col_back_sat = (sat_mul*59) / 255
         col_back_val = 40 / 255
 
-        col_text = imgui.color_convert_hsv_to_rgb(hue, 20 / 255, 235 / 255)
-        col_main = imgui.color_convert_hsv_to_rgb(hue, col_main_sat, col_main_val)
-        col_back = imgui.color_convert_hsv_to_rgb(hue, col_back_sat, col_back_val)
-        col_area = imgui.color_convert_hsv_to_rgb(hue, col_area_sat, col_area_val)
+        # col_text = imgui.color_convert_hsv_to_rgb(hue, 20 / 255, 235 / 255)
+        # col_main = imgui.color_convert_hsv_to_rgb(hue, col_main_sat, col_main_val)
+        # col_back = imgui.color_convert_hsv_to_rgb(hue, col_back_sat, col_back_val)
+        # col_area = imgui.color_convert_hsv_to_rgb(hue, col_area_sat, col_area_val)
+
+        col_text = imgui.color_convert_hsv_to_rgb([hue, 20 / 255, 235 / 255, 1.0])[:3]
+        col_main = imgui.color_convert_hsv_to_rgb([hue, col_main_sat, col_main_val, 1.0])[:3]
+        col_back = imgui.color_convert_hsv_to_rgb([hue, col_back_sat, col_back_val, 1.0])[:3]
+        col_area = imgui.color_convert_hsv_to_rgb([hue, col_area_sat, col_area_val, 1.0])[:3]
 
         style = imgui.get_style() # override active style
         imgui.style_colors_dark(style) # optional: set base colors from "Dark" (or any other) style
@@ -214,83 +228,81 @@ if imgui is not None:
         style.window_rounding = 0.0
         style.indent_spacing = 5
 
-        style.colors[imgui.COLOR_TEXT] = \
+        style.colors[imgui.Col.TEXT] = \
         [col_text[0], col_text[1], col_text[2], 1.00]
-        style.colors[imgui.COLOR_TEXT_DISABLED] = \
+        style.colors[imgui.Col.TEXT_DISABLED] = \
         [col_text[0], col_text[1], col_text[2], 0.58]
-        style.colors[imgui.COLOR_WINDOW_BACKGROUND] = \
-        [col_back[0], col_back[1], col_back[2], 1.00]
-        style.colors[imgui.COLOR_CHILD_BACKGROUND] = \
+        style.colors[imgui.Col.CHILD_BG] = \
         [col_back[0] * 0.7, col_back[1] * 0.7, col_back[2] * 0.7, 1.0]
-        style.colors[imgui.COLOR_BORDER] = \
+        style.colors[imgui.Col.BORDER] = \
         [col_text[0], col_text[1], col_text[2], 0.30]
-        style.colors[imgui.COLOR_BORDER_SHADOW] = \
+        style.colors[imgui.Col.BORDER_SHADOW] = \
         [0.0, 0.0, 0.0, 0.0]
-        style.colors[imgui.COLOR_FRAME_BACKGROUND] = \
+        style.colors[imgui.Col.FRAME_BG] = \
         [col_area[0], col_area[1], col_area[2], 1.00]
-        style.colors[imgui.COLOR_FRAME_BACKGROUND_HOVERED] = \
+        style.colors[imgui.Col.FRAME_BG_HOVERED] = \
         [col_main[0], col_main[1], col_main[2], 0.68]
-        style.colors[imgui.COLOR_FRAME_BACKGROUND_ACTIVE] = \
+        style.colors[imgui.Col.FRAME_BG_ACTIVE] = \
         [col_main[0], col_main[1], col_main[2], 1.00]
-        style.colors[imgui.COLOR_TITLE_BACKGROUND] = \
+        style.colors[imgui.Col.TITLE_BG] = \
         [col_main[0], col_main[1], col_main[2], 0.45]
-        style.colors[imgui.COLOR_TITLE_BACKGROUND_COLLAPSED] = \
+        style.colors[imgui.Col.TITLE_BG_COLLAPSED] = \
         [col_main[0], col_main[1], col_main[2], 0.35]
-        style.colors[imgui.COLOR_TITLE_BACKGROUND_ACTIVE] = \
+        style.colors[imgui.Col.TITLE_BG_ACTIVE] = \
         [col_main[0], col_main[1], col_main[2], 0.78]
-        style.colors[imgui.COLOR_MENUBAR_BACKGROUND] = \
+        style.colors[imgui.Col.MENU_BAR_BG] = \
         [col_area[0], col_area[1], col_area[2], 0.57]
-        style.colors[imgui.COLOR_SCROLLBAR_BACKGROUND] = \
+        style.colors[imgui.Col.SCROLLBAR_BG] = \
         [col_area[0], col_area[1], col_area[2], 1.00]
-        style.colors[imgui.COLOR_SCROLLBAR_GRAB] = \
+        style.colors[imgui.Col.SCROLLBAR_GRAB] = \
         [col_main[0], col_main[1], col_main[2], 0.31]
-        style.colors[imgui.COLOR_SCROLLBAR_GRAB_HOVERED] = \
+        style.colors[imgui.Col.SCROLLBAR_GRAB_HOVERED] = \
         [col_main[0], col_main[1], col_main[2], 0.78]
-        style.colors[imgui.COLOR_SCROLLBAR_GRAB_ACTIVE] = \
+        style.colors[imgui.Col.SCROLLBAR_GRAB_ACTIVE] = \
         [col_main[0], col_main[1], col_main[2], 1.00]
-        style.colors[imgui.COLOR_POPUP_BACKGROUND] = \
-        [col_area[0], col_area[1], col_area[2], 1.00]
-        style.colors[imgui.COLOR_CHECK_MARK] = \
+        style.colors[imgui.Col.POPUP_BG] = \
+        [col_area[0]*0.6, col_area[1]*0.6, col_area[2]*0.6, 1.00]
+        style.colors[imgui.Col.CHECK_MARK] = \
         [col_main[0], col_main[1], col_main[2], 0.80]
-        style.colors[imgui.COLOR_SLIDER_GRAB] = \
+        style.colors[imgui.Col.SLIDER_GRAB] = \
         [col_main[0]*0.7, col_main[1]*0.7, col_main[2]*0.7, 1.0]
-        style.colors[imgui.COLOR_SLIDER_GRAB_ACTIVE] = \
+        style.colors[imgui.Col.SLIDER_GRAB_ACTIVE] = \
         [col_main[0], col_main[1], col_main[2], 1.00]
-        style.colors[imgui.COLOR_BUTTON] = \
+        style.colors[imgui.Col.BUTTON] = \
         [col_main[0], col_main[1], col_main[2], 0.44]
-        style.colors[imgui.COLOR_BUTTON_HOVERED] = \
+        style.colors[imgui.Col.BUTTON_HOVERED] = \
         [col_main[0], col_main[1], col_main[2], 0.86]
-        style.colors[imgui.COLOR_BUTTON_ACTIVE] = \
+        style.colors[imgui.Col.BUTTON_ACTIVE] = \
         [col_main[0], col_main[1], col_main[2], 1.00]
-        style.colors[imgui.COLOR_HEADER] = \
+        style.colors[imgui.Col.HEADER] = \
         [col_main[0], col_main[1], col_main[2], 0.76]
-        style.colors[imgui.COLOR_HEADER_HOVERED] = \
+        style.colors[imgui.Col.HEADER_HOVERED] = \
         [col_main[0], col_main[1], col_main[2], 0.86]
-        style.colors[imgui.COLOR_HEADER_ACTIVE] = \
+        style.colors[imgui.Col.HEADER_ACTIVE] = \
         [col_main[0], col_main[1], col_main[2], 1.00]
-        style.colors[imgui.COLOR_SEPARATOR] = \
+        style.colors[imgui.Col.SEPARATOR] = \
         [col_text[0], col_text[1], col_text[2], 0.32]
-        style.colors[imgui.COLOR_SEPARATOR_HOVERED] = \
+        style.colors[imgui.Col.SEPARATOR_HOVERED] = \
         [col_text[0], col_text[1], col_text[2], 0.78]
-        style.colors[imgui.COLOR_SEPARATOR_ACTIVE] = \
+        style.colors[imgui.Col.SEPARATOR_ACTIVE] = \
         [col_text[0], col_text[1], col_text[2], 1.00]
-        style.colors[imgui.COLOR_RESIZE_GRIP] = \
+        style.colors[imgui.Col.RESIZE_GRIP] = \
         [col_main[0], col_main[1], col_main[2], 0.20]
-        style.colors[imgui.COLOR_RESIZE_GRIP_HOVERED] = \
+        style.colors[imgui.Col.RESIZE_GRIP_HOVERED] = \
         [col_main[0], col_main[1], col_main[2], 0.78]
-        style.colors[imgui.COLOR_RESIZE_GRIP_ACTIVE] = \
+        style.colors[imgui.Col.RESIZE_GRIP_ACTIVE] = \
         [col_main[0], col_main[1], col_main[2], 1.00]
-        style.colors[imgui.COLOR_PLOT_LINES] = \
+        style.colors[imgui.Col.PLOT_LINES] = \
         [col_text[0], col_text[1], col_text[2], 0.63]
-        style.colors[imgui.COLOR_PLOT_LINES_HOVERED] = \
+        style.colors[imgui.Col.PLOT_LINES_HOVERED] = \
         [col_main[0], col_main[1], col_main[2], 1.00]
-        style.colors[imgui.COLOR_PLOT_HISTOGRAM] = \
+        style.colors[imgui.Col.PLOT_HISTOGRAM] = \
         [col_text[0], col_text[1], col_text[2], 0.63]
-        style.colors[imgui.COLOR_PLOT_HISTOGRAM_HOVERED] = \
+        style.colors[imgui.Col.PLOT_HISTOGRAM_HOVERED] = \
         [col_main[0], col_main[1], col_main[2], 1.00]
-        style.colors[imgui.COLOR_TEXT_SELECTED_BACKGROUND] = \
+        style.colors[imgui.Col.TEXT_SELECTED_BG] = \
         [col_main[0], col_main[1], col_main[2], 0.43]
-        style.colors[imgui.COLOR_MODAL_WINDOW_DIM_BACKGROUND] = \
+        style.colors[imgui.Col.MODAL_WINDOW_DIM_BG] = \
         [0.20, 0.20, 0.20, 0.35]
 
 
@@ -298,12 +310,16 @@ if imgui is not None:
         if isinstance(val, bool):
             return 'checkbox'
         elif isinstance(val, int):
+            if 'type' in opts:
+                return opts['type']
             if 'min' in opts and 'max' in opts:
                 return 'int_slider'
             elif 'selection' in opts:
                 return 'selection'
             return 'int'
         elif isinstance(val, float):
+            if 'type' in opts:
+                return opts['type']
             if 'min' in opts and 'max' in opts:
                 return 'float_slider'
             return 'float'
@@ -329,6 +345,9 @@ if imgui is not None:
             self.changed = set()
             self.cur_preset_name = ''
 
+        def set_theme(self, hue):
+            set_theme(hue)
+
         def force_changed(self, params, gui_params, parent=''):
             for name, val in gui_params.items():
                 if name == '__key__':
@@ -349,7 +368,7 @@ if imgui is not None:
                     continue
                 if type(val) == dict: # A dict marks a group
                     key = val['__key__'] # Assume this has been filled automatically
-                    if imgui.tree_node(name, imgui.TREE_NODE_DEFAULT_OPEN):
+                    if imgui.tree_node(name, imgui.TreeNodeFlags.DEFAULT_OPEN):
                         imgui.push_id(name)
                         self.show_params(params[key], val, parent + key + '.', depth+1)
                         imgui.pop_id()
@@ -367,7 +386,12 @@ if imgui is not None:
                     try:
                         changed = False
                         if param_type == 'int':
-                            changed, params[key] = imgui.input_int(name, params[key])
+                            changed, params[key] = imgui.input_int(name, int(params[key]))
+                            if 'min' in opts:
+                                params[key] = max(params[key], opts['min'])
+                            if 'max' in opts:
+                                params[key] = min(params[key], opts['max'])
+
                         elif param_type == 'button':
                             if imgui.button(name):
                                 val()
@@ -376,9 +400,9 @@ if imgui is not None:
                             if 'buf_length' in opts:
                                 buf_length = opts['buf_length']
                             if 'multiline' in opts and opts['multiline']:
-                                changed, params[key] = imgui.input_text_multiline(name, params[key], buf_length, 0, 0, imgui.INPUT_TEXT_ENTER_RETURNS_TRUE)
+                                changed, params[key] = imgui.input_text_multiline(name, params[key], [0, 0], imgui.InputTextFlags.ENTER_RETURNS_TRUE)
                             else:
-                                changed, params[key] = imgui.input_text(name, params[key], buf_length, imgui.INPUT_TEXT_ENTER_RETURNS_TRUE)
+                                changed, params[key] = imgui.input_text(name, params[key],  imgui.InputTextFlags.ENTER_RETURNS_TRUE)
                         elif param_type == 'selection':
                             changed, params[key] = imgui.combo(name, params[key], opts['selection'])
                         elif param_type == 'float':
@@ -386,16 +410,16 @@ if imgui is not None:
                         elif param_type == 'float_slider':
                             changed, params[key] = imgui.slider_float(name, params[key], opts['min'], opts['max'])
                         elif param_type == 'int_slider':
-                            changed, params[key] = imgui.slider_int(name, params[key], opts['min'], opts['max'])
+                            changed, params[key] = imgui.slider_int(name, int(params[key]), int(opts['min']), int(opts['max']))
                         elif param_type == 'checkbox':
                             changed, params[key] = imgui.checkbox(name, params[key])
                         elif param_type == 'color':
                             clr = np.array(params[key], dtype=np.float32)
                             clr /= self.sketch.canvas.color_scale[:len(clr)]
                             if len(clr) == 3:
-                                changed, clr = imgui.color_edit3(name, *clr) #, imgui.COLOR_EDIT_DISPLAY_HSV)
+                                changed, clr = imgui.color_edit3(name, clr) #, imgui.COLOR_EDIT_DISPLAY_HSV)
                             else:
-                                changed, clr = imgui.color_edit4(name, *clr) #, imgui.COLOR_EDIT_DISPLAY_HSV)
+                                changed, clr = imgui.color_edit4(name, clr) #, imgui.COLOR_EDIT_DISPLAY_HSV)
                             params[key] = np.array(clr)*self.sketch.canvas.color_scale[:len(clr)]
                         if changed:
                             self.changed.add(parent + key)
@@ -413,12 +437,13 @@ if imgui is not None:
             self.sketch = sketch
             # Top bar
             ratio = sketch.get_pixel_ratio()
-            imgui.set_next_window_size(sketch.window_width, sketch.toolbar_height)
-            imgui.set_next_window_position(0, 0)
-            imgui.begin("Toolbar", True, (imgui.WINDOW_NO_RESIZE |
-                                            imgui.WINDOW_NO_TITLE_BAR |
-                                            imgui.WINDOW_NO_SAVED_SETTINGS |
-                                            imgui.WINDOW_NO_SCROLLBAR))
+            imgui.set_next_window_size([sketch.window_width, sketch.toolbar_height])
+            imgui.set_next_window_pos([0, 0])
+            imgui.begin("Toolbar", True, (imgui.WindowFlags.NO_RESIZE |
+                                            imgui.WindowFlags.NO_TITLE_BAR |
+                                            #imgui.WindowFlags.MENU_BAR |
+                                            imgui.WindowFlags.NO_SAVED_SETTINGS |
+                                            imgui.WindowFlags.NO_SCROLLBAR))
 
             if imgui.button('Load sketch...'):
                 path = sketch.open_file_dialog('py')
@@ -434,59 +459,140 @@ if imgui is not None:
                     if os.path.isfile(json_path):
                         shutil.copy(json_path, path.replace('.py', '.json'))
             if imgui.is_item_hovered():
-                with imgui.begin_tooltip():
+                if imgui.begin_tooltip():
                     imgui.text('Save a copy of the current sketch')
                     imgui.text('and its parameters')
+                    imgui.end_tooltip()
             imgui.same_line()
             if imgui.button('Reload'):
                 sketch.must_reload = True
             if imgui.is_item_hovered():
-                with imgui.begin_tooltip():
-                    imgui.text('Force reload sketch')
+                imgui.set_tooltip('Force reload sketch')
             imgui.same_line()
-            if imgui.button('Save SVG...'):
+            if imgui.button('Save...'):
+                imgui.open_popup('Save popup')
+            if imgui.begin_popup('Save popup'):
                 name = os.path.splitext(os.path.basename(sketch.path))[0]
-                path = sketch.save_file_dialog('svg', filename=name)
+                path = ''
+                if imgui.menu_item('Pdf...')[0]:
+                    path = sketch.save_file_dialog('pdf', filename=name) #['pdf', 'svg', 'png'], filename=name)
+                elif imgui.menu_item('Svg...')[0]:
+                    path = sketch.save_file_dialog('svg', filename=name) #['pdf', 'svg', 'png'], filename=name)
+                elif imgui.menu_item('Png...')[0]:
+                    path = sketch.save_file_dialog('png', filename=name) #['pdf', 'svg', 'png'], filename=name)
                 if path:
                     sketch.dump_canvas(path)
+                if imgui.menu_item('Mp4...')[0]:
+                    path = sketch.save_file_dialog('mp4', filename=name) #['pdf', 'svg', 'png'], filename=name)
+                    if path:
+                        sketch.grab_movie(path)
+                imgui.end_popup()
             imgui.same_line()
-            if imgui.button('Save PDF...'):
-                name = os.path.splitext(os.path.basename(sketch.path))[0]
-                path = sketch.save_file_dialog('pdf', filename=name)
-                if path:
-                    sketch.dump_canvas(path)
-            if imgui.is_item_hovered():
-                with imgui.begin_tooltip():
-                    imgui.text('Save sketch output as SVG')
+            if imgui.button('Settings...'):
+                imgui.open_popup('Settings popup')
+            if imgui.begin_popup('Settings popup'):
+                imgui.separator_text('Window')
+                changed, flag = imgui.checkbox('Always on top', sketch.settings['floating_window'])
+                if changed:
+                    sketch.set_floating(flag)
+                changed, flag = imgui.checkbox('Startup with toolbar', sketch.settings['show_toolbar'])
+                if changed:
+                    sketch.settings['show_toolbar'] = flag
+
+                imgui.separator_text('Animation')
+                nf = sketch.settings['num_movie_frames']
+                changed, nf = imgui.input_int('num frames', nf)
+                nf = max(1, nf)
+                if changed:
+                    sketch.settings['num_movie_frames'] = nf
+                fps = sketch._fps
+                if fps > 0:
+                    dur = (1.0/fps)*nf
+                    imgui.text(f'{np.round(dur, 1)} seconds for {fps} fps')
+                else:
+                    imgui.text('No framerate set')
+
+                if sketch.osc_enabled:
+                    imgui.separator_text('OSC')
+                    changed, res = imgui.input_text('Recv. port',
+                                                   str(sketch.settings['osc']['recv_port']),
+                                                   flags=imgui.InputTextFlags.ENTER_RETURNS_TRUE)
+                    if changed:
+                        try:
+                            res = int(res)
+                            sketch.settings['osc']['recv_port'] = res
+                            sketch.start_osc()
+                        except ValueError:
+                            print("Port must be an integer")
+
+                        
+                    changed, res = imgui.input_text('Send addr.',
+                                                    str(sketch.settings['osc']['send_addr']),
+                                                    imgui.InputTextFlags.ENTER_RETURNS_TRUE)
+                    if changed:
+                        sketch.settings['osc']['send_addr'] = res
+                        sketch.start_osc()
+                    changed, res = imgui.input_text('Send port',
+                                                   str(sketch.settings['osc']['send_port']),
+                                                   flags=imgui.InputTextFlags.ENTER_RETURNS_TRUE)
+                    if changed:
+                        try:
+                            res = int(res)
+                            sketch.settings['osc']['send_port'] = res
+                            sketch.start_osc()
+                        except ValueError:
+                            print("Port must be an integer")
+
+                imgui.end_popup()
+
+            # if imgui.button('Save Canvas...'):
+
+            #     path = sketch.save_file_dialog(['pdf', 'svg'], filename=name) #['pdf', 'svg', 'png'], filename=name)
+
+            #     if path:
+            #         sketch.dump_canvas(path)
+            # if imgui.is_item_hovered():
+            #     imgui.set_tooltip('Save sketch output as an image or vector file')
+
+            # imgui.same_line()
+            # if imgui.button('Save PDF...'):
+            #     name = os.path.splitext(os.path.basename(sketch.path))[0]
+            #     path = sketch.save_file_dialog('pdf', filename=name)
+            #     if path:
+            #         sketch.dump_canvas(path)
 
-            imgui.same_line()
-            imgui.push_style_color(imgui.COLOR_TEXT, 0.5, 0.5, 0.5)
-            script_name = os.path.basename(sketch.path)
-            imgui.text('Sketch: ' + script_name)
-            imgui.pop_style_color(1)
             imgui.end()
 
         def begin_gui(self, sketch):
             self.sketch = sketch
             ratio = 1 #sketch.window.get_pixel_ratio()
-            imgui.set_next_window_size(self.width, (sketch.window_height - sketch.toolbar_height)*ratio)
-            imgui.set_next_window_position((sketch.window_width - self.width)*ratio, sketch.toolbar_height)
-            imgui.begin("Py5sketch", True, (imgui.WINDOW_NO_RESIZE |
-                                            imgui.WINDOW_NO_TITLE_BAR |
-                                            imgui.WINDOW_NO_SAVED_SETTINGS))
+            imgui.set_next_window_size([self.width, (sketch.window_height - sketch.toolbar_height)*ratio])
+            imgui.set_next_window_pos([(sketch.window_width - self.width)*ratio, sketch.toolbar_height])
+            imgui.begin("Py5sketch", True, (imgui.WindowFlags.NO_RESIZE |
+                                            imgui.WindowFlags.NO_TITLE_BAR |
+                                            imgui.WindowFlags.NO_SAVED_SETTINGS))
             imgui.begin_child("Sketch")
+            imgui.push_style_color(imgui.Col.TEXT, [0.5, 0.5, 0.5])
+            script_name = os.path.basename(sketch.path)
+            imgui.text_wrapped('Sketch: ' + script_name)
+            imgui.pop_style_color(1)
+            if sketch.desc:
+                imgui.text_wrapped(sketch.desc)
 
         def show_sketch_controls(self):
-            return imgui.collapsing_header("Controls", None, imgui.TREE_NODE_DEFAULT_OPEN)[0]
+            return imgui.collapsing_header("Controls", None, imgui.TreeNodeFlags.DEFAULT_OPEN)[0]
+
+        def clear_changed(self):
+            self.changed = set()
 
         def from_params(self, sketch, callback=None, init=True):
             self.sketch = sketch
-            self.changed = set()
+            # self.changed = set() 
             if init:
                 self.begin_gui()
             
 
-            if imgui.collapsing_header("Parameters", None, imgui.TREE_NODE_DEFAULT_OPEN)[0]:
+            if imgui.collapsing_header("Parameters", None, imgui.TreeNodeFlags.DEFAULT_OPEN)[0]:
                 if sketch.params is not None:
                     self.show_params(sketch.params.params, sketch.params.gui_params)
 
@@ -495,7 +601,7 @@ if imgui is not None:
             imgui.spacing()
 
             if sketch.params is not None:
-                if imgui.collapsing_header("Presets", None, imgui.TREE_NODE_DEFAULT_OPEN):
+                if imgui.collapsing_header("Presets", None, imgui.TreeNodeFlags.DEFAULT_OPEN):
                     preset_names = [k for k in sketch.params.presets.keys()]
                     preset_name = ''
                     if 0 <= sketch.params.current_preset < len(preset_names):
@@ -504,17 +610,21 @@ if imgui is not None:
                     buttons = False
                     if self.cur_preset_name:
                         buttons = True
-                        if imgui.button('+'):
+                        label = '+'
+                        if self.cur_preset_name in preset_names:
+                            label = 'v'
+                        if imgui.button(label):
                             sketch.params.add_preset(self.cur_preset_name)
                             preset_names = [k for k in sketch.params.presets.keys()]
                             sketch.current_preset = sketch.params.preset_index(self.cur_preset_name)
                             preset_name = self.cur_preset_name
                         if imgui.is_item_hovered():
-                            with imgui.begin_tooltip():
+                            if imgui.begin_tooltip():
                                 if self.cur_preset_name == preset_name:
                                     imgui.text('Update preset')
                                 else:
                                     imgui.text('Add new preset')
+                                imgui.end_tooltip()
 
                     if preset_name:
                         if buttons:
@@ -524,19 +634,17 @@ if imgui is not None:
                             sketch.params.delete_preset(preset_name)
                             sketch.params.current_preset = -1
                         if imgui.is_item_hovered():
-                            with imgui.begin_tooltip():
-                                imgui.text('Delete selected preset')
+                            imgui.set_tooltip('Delete selected preset')
 
                     if buttons:
                         imgui.same_line()
-                    changed, self.cur_preset_name = imgui.input_text('Name', self.cur_preset_name, 512, imgui.INPUT_TEXT_ENTER_RETURNS_TRUE)
+                    changed, self.cur_preset_name = imgui.input_text('Name', self.cur_preset_name, imgui.InputTextFlags.ENTER_RETURNS_TRUE)
                     if preset_name:
                         if imgui.is_item_hovered():
-                            with imgui.begin_tooltip():
-                                imgui.text('Press enter to update preset name')
+                            imgui.set_tooltip('Press enter to update preset name')
 
 
-                    clicked, sketch.params.current_preset = imgui.listbox("Presets", sketch.params.current_preset, preset_names)
+                    clicked, sketch.params.current_preset = imgui.list_box("Presets", sketch.params.current_preset, preset_names)
 
                     if clicked:
                         sketch.params.apply_preset(preset_names[sketch.params.current_preset])
